@@ -51,6 +51,9 @@ DEFINE_bool(baidu_protocol_use_fullname, true,
             "If this flag is true, baidu_std puts service.full_name in requests"
             ", otherwise puts service.name (required by jprotobuf).");
 
+DEFINE_bool(baidu_std_protocol_deliver_timeout_ms, false,
+            "If this flag is true, baidu_std puts timeout_ms in requests.");
+
 // Notes:
 // 1. 12-byte header [PRPC][body_size][meta_size]
 // 2. body_size and meta_size are in network byte order
@@ -228,10 +231,12 @@ void SendRpcResponse(int64_t correlation_id,
     if (span) {
         span->set_response_size(res_buf.size());
     }
-    if (stream_ptr) {
-        CHECK(accessor.remote_stream_settings() != NULL);
+    // Send rpc response over stream even if server side failed to create
+    // stream for some reasons.
+    if(cntl->has_remote_stream()){
         // Send the response over stream to notify that this stream connection
         // is successfully built.
+        // Response_stream can be INVALID_STREAM_ID when error occurs.
         if (SendStreamData(sock, &res_buf,
                            accessor.remote_stream_settings()->stream_id(),
                            accessor.response_stream()) != 0) {
@@ -239,13 +244,18 @@ void SendRpcResponse(int64_t correlation_id,
             PLOG_IF(WARNING, errcode != EPIPE) << "Fail to write into " << *sock;
             cntl->SetFailed(errcode, "Fail to write into %s",
                             sock->description().c_str());
-            ((Stream*)stream_ptr->conn())->Close();
+            if(stream_ptr) {
+                ((Stream*)stream_ptr->conn())->Close();
+            }
             return;
         }
-        // Now it's ok the mark this server-side stream as connectted as all the
-        // written user data would follower the RPC response.
-        ((Stream*)stream_ptr->conn())->SetConnected();
-    } else {
+
+        if(stream_ptr) {
+            // Now it's ok the mark this server-side stream as connectted as all the
+            // written user data would follower the RPC response.
+            ((Stream*)stream_ptr->conn())->SetConnected();
+        }
+    } else{
         // Have the risk of unlimited pending responses, in which case, tell
         // users to set max_concurrency.
         Socket::WriteOptions wopt;
@@ -346,6 +356,9 @@ void ProcessRpcRequest(InputMessageBase* msg_base) {
     if (request_meta.has_request_id()) {
         cntl->set_request_id(request_meta.request_id());
     }
+    if (request_meta.has_timeout_ms()) {
+        cntl->set_timeout_ms(request_meta.timeout_ms());
+    }
     cntl->set_request_compress_type((CompressType)meta.compress_type());
     accessor.set_server(server)
         .set_security_mode(security_mode)
@@ -487,7 +500,7 @@ void ProcessRpcRequest(InputMessageBase* msg_base) {
                 req.get(), res.get(), server,
                 method_status, msg->received_us());
 
-        // optional, just release resourse ASAP
+        // optional, just release resource ASAP
         msg.reset();
         req_buf.clear();
 
@@ -552,18 +565,21 @@ void ProcessRpcResponse(InputMessageBase* msg_base) {
 
     const bthread_id_t cid = { static_cast<uint64_t>(meta.correlation_id()) };
     Controller* cntl = NULL;
+
+    StreamId remote_stream_id = meta.has_stream_settings() ? meta.stream_settings().stream_id(): INVALID_STREAM_ID;
+
     const int rc = bthread_id_lock(cid, (void**)&cntl);
     if (rc != 0) {
         LOG_IF(ERROR, rc != EINVAL && rc != EPERM)
             << "Fail to lock correlation_id=" << cid << ": " << berror(rc);
-        if (meta.has_stream_settings()) {
+        if (remote_stream_id != INVALID_STREAM_ID) {
             SendStreamRst(msg->socket(), meta.stream_settings().stream_id());
         }
         return;
     }
     
     ControllerPrivateAccessor accessor(cntl);
-    if (meta.has_stream_settings()) {
+    if (remote_stream_id != INVALID_STREAM_ID) {
         accessor.set_remote_stream_settings(
                 new StreamSettings(meta.stream_settings()));
     }
@@ -615,7 +631,7 @@ void ProcessRpcResponse(InputMessageBase* msg_base) {
     } while (0);
     // Unlocks correlation_id inside. Revert controller's
     // error code if it version check of `cid' fails
-    msg.reset();  // optional, just release resourse ASAP
+    msg.reset();  // optional, just release resource ASAP
     accessor.OnResponse(cid, saved_error);
 }
 
@@ -672,6 +688,13 @@ void PackRpcRequest(butil::IOBuf* req_buf,
     if (attached_size) {
         meta.set_attachment_size(attached_size);
     }
+
+    if (FLAGS_baidu_std_protocol_deliver_timeout_ms) {
+        if (accessor.real_timeout_ms() > 0) {
+            request_meta->set_timeout_ms(accessor.real_timeout_ms());
+        }
+    }
+
     Span* span = accessor.span();
     if (span) {
         request_meta->set_trace_id(span->trace_id());
